@@ -12,6 +12,7 @@ def get_stepback_insight_openai(
     base_url = (
         "https://api.perplexity.ai" if model_source == "perplexity" 
         else "https://router.huggingface.co/v1" if model_source == "huggingface"
+        else "https://api.x.ai/v1" if model_source == "xai"
         else None
     )
     
@@ -188,6 +189,7 @@ def chain_of_verification_openai(
         final_response = client.chat.completions.create(
             model=user_model,
             messages=[{'role': 'user', 'content': step4_filled}],
+            response_format={"type": "json_object"},
             **({"temperature": creativity} if creativity is not None else {})
         )
         
@@ -261,15 +263,29 @@ def chain_of_verification_anthropic(
             .replace('<<VERIFICATION_QA>>', verification_qa_text))
         
         print(f"Final prompt:\n{step4_filled}\n")
+
+        tools = [{
+            "name": "return_categories",
+            "description": "Return categorization results as 0 (not present) or 1 (present) for each category",
+            "input_schema": {
+                "type": "object",
+                "properties": properties,
+                "required": list(properties.keys())  # All categories required
+            }
+        }]
         
         final_response = client.messages.create(
             model=user_model,
             messages=[{'role': 'user', 'content': step4_filled}],
             max_tokens=4096,
+            tools=tools,
+            tool_choice={"type": "tool", "name": "return_categories"},
             **({"temperature": creativity} if creativity is not None else {})
         )
         
-        verified_reply = final_response.content[0].text
+        result_dict = final_response.content[0].input
+
+        verified_reply = json.dumps(result_dict)
         print("Chain of verification completed. Final response generated.\n")
         
         return verified_reply
@@ -305,7 +321,8 @@ def chain_of_verification_google(
             "contents": [{
                 "parts": [{"text": step2_filled}]
             }],
-            **({"generationConfig": {"temperature": creativity}} if creativity is not None else {})
+            **({"generationConfig": {"temperature": creativity}} if creativity is not None else {}),
+            **({"thinkingConfig": {"thinkingBudget": thinking_budget}} if thinking_budget is not None else {})
         }
         
         result_step2 = make_google_request(url, headers, payload_step2)
@@ -327,7 +344,8 @@ def chain_of_verification_google(
                 "contents": [{
                     "parts": [{"text": step3_filled}]
                 }],
-                **({"generationConfig": {"temperature": creativity}} if creativity is not None else {})
+                **({"generationConfig": {"temperature": creativity}} if creativity is not None else {}),
+                **({"thinkingConfig": {"thinkingBudget": thinking_budget}} if thinking_budget is not None else {})
             }
             
             result_step3 = make_google_request(url, headers, payload_step3)
@@ -346,7 +364,11 @@ def chain_of_verification_google(
             "contents": [{
                 "parts": [{"text": step4_filled}]
             }],
-            **({"generationConfig": {"temperature": creativity}} if creativity is not None else {})
+            "generationConfig": {
+                                "responseMimeType": "application/json",
+                                **({"temperature": creativity} if creativity is not None else {}),
+                                **({"thinkingConfig": {"thinkingBudget": thinking_budget}} if thinking_budget is not None else {})
+            }
         }
         
         result_step4 = make_google_request(url, headers, payload_step4)
@@ -419,6 +441,7 @@ def chain_of_verification_mistral(
         final_response = client.chat.complete(
             model=user_model,
             messages=[{'role': 'user', 'content': step4_filled}],
+            response_format={"type": "json_object"},
             **({"temperature": creativity} if creativity is not None else {})
         )
         
@@ -450,6 +473,7 @@ def get_openai_top_n(
     base_url = (
         "https://api.perplexity.ai" if model_source == "perplexity" 
         else "https://router.huggingface.co/v1" if model_source == "huggingface"
+        else "https://api.x.ai/v1" if model_source == "xai"
         else None
     )
 
@@ -468,7 +492,6 @@ def get_openai_top_n(
     
     return response_obj.choices[0].message.content
 
-# anthropic explore corpus call
 def get_anthropic_top_n(
     prompt,
     user_model,
@@ -484,7 +507,7 @@ def get_anthropic_top_n(
     import anthropic
     client = anthropic.Anthropic(api_key=api_key)
 
-    # build system prompt
+    # Build system prompt
     if research_question:
         system_content = (f"You are a helpful assistant that extracts categories from survey responses. "
                         f"The specific task is to identify {specificity} categories of responses to a survey question. "
@@ -502,6 +525,4 @@ def get_anthropic_top_n(
         **({"temperature": creativity} if creativity is not None else {})
     )
     
-    return response_obj.content[0].text
-
-
+    return response_obj.content[0].text
\ No newline at end of file
